/**********************************************************************
* nrf24l01 lib                                                        *
*                                                                     *
* Copyright (c) Davide Gironi, 2012                                   *
* http://davidegironi.blogspot.com/                                   *
*                                                                     *
* References:                                                         *
*    This library is based upon nRF24L01 avr lib by Stefan Engelke    *
*    http://www.tinkerer.eu/AVRLib/nRF24L01                           *
**********************************************************************/

/**********************************************************************
* This library is free software; you can redistribute it and/or       *
* modify it under the terms of the GNU Lesser General Public          *
* License as published by the Free Software Foundation.               *
*                                                                     *
* This library is distributed in the hope that it will be useful,     *
* but WITHOUT ANY WARRANTY; without even the implied warranty of      *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   *
* Lesser General Public License for more details.                     *
*                                                                     *
* You should have received a copy of the GNU Lesser General Public    *
* License along with this library; if not, write to the               *
* Free Software Foundation, Inc.,                                     *
* 59 Temple Place, Suite 330,                                         *
* Boston, MA                                                          *
* 02111-1307                                                          *
* USA                                                                 *
*                                                                     *
* For further information, visit www.gnu.org
* http://www.tinkerer.eu/AVRLib/nRF24L01/                       *
**********************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "mirf.h"
#include "nRF24L01.h"

//uint8_t rxaddr[5] = {0xE7, 0xE7, 0xE7, 0xE7, 0x11};
//uint8_t txaddr[5] = {0xE7, 0xE7, 0xE7, 0xE7, 0x11};

uint8_t self_addr[] = {0xC2, 0xC2, 0xC2, 0xC2, 0xC2}; //  
uint8_t remote_addr[] = {0xC2, 0xC2, 0xC2, 0xC2, 0xC2}; //   

uint8_t ACK_OK = 0;

//initialize spi
void spi_init()
{
    DDR_SPI &= ~( (1<<SPI_MOSI) | (1<<SPI_MISO) | (1<<SPI_SS) | (1<<SPI_SCK) );
    DDR_SPI |= ( (1<<SPI_MOSI) | (1<<SPI_SS) | (1<<SPI_SCK) );

    SPCR = ((1<<SPE)|               // SPI Enable
            (0<<SPIE)|              // SPI Interupt Enable
            (0<<DORD)|              // Data Order (0:MSB first / 1:LSB first)
            (1<<MSTR)|              // Master/Slave select
            (0<<SPR1)|(1<<SPR0)|    // SPI Clock Rate foc/8
			//(1<<SPR1)|(1<<SPR0)|    // SPI Clock Rate foc/128
            (0<<CPOL)|              // Clock Polarity (0:SCK low / 1:SCK hi when idle)
            (0<<CPHA));             // Clock Phase (0:leading / 1:trailing edge sampling)

    SPSR = (1<<SPI2X); // double spi speed bit
}

//spi write one byte and read it back
uint8_t spi_writeread(uint8_t data)
{
    SPDR = data;
    while((SPSR & (1<<SPIF)) == 0);
    return SPDR;
}

//initialize mirf
void mirf_init() 
{
    DDRB |= ( (1<<CSN) | (1<<CE) );

    mirf_CE_lo;
    mirf_CSN_hi;

    spi_init();
}


/*
CONFIG: PRIM_RX=0, PWR_UP=1
EN_AA: ENAA_P0=1
EN_RXADDR: ERX_P0=1
SETUP_RETR: ARD=0b10 (750us), ARC=3
RF_CH: 100
RF_SETUP: RF_DR_LOW=0, RF_DR_HIGH=0, RF_PWR=0b11
RX_ADDR_P0: 0x11,0x22,0x33,0x44,0x55
TX_ADDR: 0x11,0x22,0x33,0x44,0x55
*/
//config mirf
uint8_t mirf_config()
{

	  for(uint8_t cnt = 100;;) {
		  mirf_write_register(CONFIG, (1 << EN_CRC) | (1 << CRCO) | (1 << PRIM_RX)); //  
	    if (mirf_read_register(CONFIG) == ((1 << EN_CRC) | (1 << CRCO) | (1 << PRIM_RX)))
	      break;
	    //      ,    -  ,   .
	    if (!cnt--)
	      return 0; //   100      ,    
	    _delay_ms(1);
	  }

	_delay_ms(50);
/*
	mirf_write_register(EN_RXADDR, (1 << ERX_P0) | (1 << ERX_P1)); //   0  1
    mirf_write_register(RF_CH, mirf_CH);			//set RF channel
    if (mirf_PAYLOAD!=0){
    	//     
    	mirf_write_register(RX_PW_P0, mirf_PAYLOAD); //length of incoming payload
    	mirf_write_register(FEATURE, (mirf_read_register(FEATURE) & ~0x01)); //      
    }else{
	//       
    	mirf_write_register(FEATURE,(1 << EN_DPL) | (1<<EN_ACK_PAY) | (1<<EN_DYN_ACK));
    	mirf_write_register(DYNPD, (1 << DPL_P0)); 	//      0
    	mirf_write_register(EN_AA,(1<<ENAA_P0));	//      0
	}

    mirf_send_cmd(FLUSH_TX);	//    
    mirf_send_cmd(FLUSH_RX);	//    

    mirf_setRX; // rx mode
*/
	mirf_write_register(EN_AA, (1 << ENAA_P1)); 												//      1
	mirf_write_register(EN_RXADDR, (1 << ERX_P0) | (1 << ERX_P1)); 								//   0  1
	mirf_write_register(SETUP_AW, SETUP_AW_5BYTES_ADDRESS); 									//    5 
	mirf_write_register(SETUP_RETR, SETUP_RETR_DELAY_500MKS | SETUP_RETR_UP_TO_2_RETRANSMIT);	//      500       
	mirf_write_register(RF_CH, mirf_CH); 														//   
	mirf_write_register(RF_SETUP, RF_SETUP_2MBPS | RF_SETUP_0DBM); 								//   2 /   0dBm

	mirf_write_registers(RX_ADDR_P0, remote_addr, 5);											//         0
	mirf_write_registers(TX_ADDR, remote_addr, 5);												//   
	//mirf_write_registers(RX_ADDR_P1, self_addr, 5);												//   

	//mirf_write_register(RX_PW_P0, 0);
	//mirf_write_register(RX_PW_P1, 32);
	mirf_write_register(DYNPD, (1 << DPL_P0) | (1 << DPL_P1)); 									//      0  1
	mirf_write_register(FEATURE,(1 << EN_DPL) | (1<<EN_ACK_PAY) | (1<<EN_DYN_ACK)); 			//     

	mirf_send_cmd(FLUSH_TX);																	//    
	mirf_send_cmd(FLUSH_RX);																	//    

	mirf_write_register(CONFIG, (1 << EN_CRC) | (1 << CRCO) | (1 << PWR_UP) | (1 << PRIM_RX)); 	//      
    _delay_us(135);																				//    ( )
    mirf_CE_hi;																					//   NRF
    return 1;
}


//set tx address
void mirf_set_txaddr(uint8_t *addr)
{
    mirf_write_registers(TX_ADDR, addr, 5);
}

//get status
uint8_t mirf_get_status()
{
	uint8_t status = 0;
	mirf_CSN_lo;
	status = spi_writeread(NOP);
	mirf_CSN_hi;
	return status;
}

//read one register
uint8_t mirf_read_register(uint8_t reg)
{
	mirf_CSN_lo;
	spi_writeread(R_REGISTER | (REGISTER_MASK & reg));
    uint8_t result = spi_writeread(NOP);
    mirf_CSN_hi;
    return result;
}

//read many registers
void mirf_read_registers(uint8_t reg, uint8_t *value, uint8_t len)
{
    mirf_CSN_lo;
    spi_writeread(R_REGISTER | (REGISTER_MASK & reg));
	for (uint8_t i = 0; i < len; i++)
		value[i] = spi_writeread(NOP);
    mirf_CSN_hi;
}

//write one register
void mirf_write_register(uint8_t reg, uint8_t value)
{
	mirf_CSN_lo;
	spi_writeread(W_REGISTER | (REGISTER_MASK & reg));
	spi_writeread(value);
	mirf_CSN_hi;
}

//write many registers
void mirf_write_registers(uint8_t reg, uint8_t *value, uint8_t len)
{
    mirf_CSN_lo;
    spi_writeread(W_REGISTER | (REGISTER_MASK & reg));
	for (uint8_t i = 0; i < len; i++)
		 spi_writeread(value[i]);
    mirf_CSN_hi;
}

//check if there is rx data
uint8_t mirf_read_ready()
{
    uint8_t status = mirf_get_status();
    return status & (1<<RX_DR);
}

//read data
void mirf_read(uint8_t *data)
{
	//read rx register
    mirf_CSN_lo;
    spi_writeread( R_RX_PAYLOAD );
    for (uint8_t i = 0; i < mirf_PAYLOAD; i++)
    	data[i] = spi_writeread(NOP);
    mirf_CSN_hi;
    //reset register
    mirf_write_register(STATUS,(1<<RX_DR));
}

uint8_t nrf24_retransmissionCount()
{
    
    
    return (mirf_read_register(OBSERVE_TX)&0x0F);
}

uint8_t nrf24_isSending()
{

    /* if sending successful (TX_DS) or max retries exceded (MAX_RT). */
    if((mirf_get_status()&((1 << TX_DS)|(1 << MAX_RT))))
    {        
        return 0; /* false */
    }

    return 1; /* true */

}

uint8_t nrf24_lastMessageStatus()
{
    uint8_t rv;

    rv = mirf_get_status();

    /* Transmission went OK */
    if((rv & ((1 << TX_DS))))
    {
        return 1;
    }
    /* Maximum retransmission count is reached */
    /* Last message probably went missing ... */
    else if((rv & ((1 << MAX_RT))))
    {
        return 2;
    }  
    /* Probably still sending ... */
    else
    {
        return 0xFF;
    }
}

/**
 *   NRF  
 */
void set_TX(void){
	uint8_t conf = mirf_read_register(CONFIG);
	mirf_write_register(CONFIG, (conf & ~(1 << PRIM_RX)));	//  NRF     PRIM_RX
}
/**
 *
return 1 - Transmission went OK
return 2 - Maximum retransmission count is reached
return 0xFF - Probably still sending
return 3 - powerdown
 */
//write data
uint8_t mirf_write(char *data, uint8_t data_lenght)
{
	  uint8_t conf = mirf_read_register(CONFIG);
	  if (!(conf & (1 << PWR_UP))){ //    -  ,   
		  printf_P(PSTR("CONFIG1\t\t = %02d MASK_RX_DR=%x MASK_TX_DS=%x MASK_MAX_RT=%x EN_CRC=%x CRCO=%x PWR_UP=%x PRIM_RX=%x\r"),
				  conf,
		        	(conf & _BV(MASK_RX_DR))?1:0,
		        	(conf & _BV(MASK_TX_DS))?1:0,
		            (conf & _BV(MASK_MAX_RT))?1:0,
		            (conf & _BV(EN_CRC))?1:0,
		            (conf & _BV(CRCO))?1:0,
		            (conf & _BV(PWR_UP))?1:0,
		            (conf & _BV(PRIM_RX))?1:0
		  );
		  return 3;
	  }
	  conf = mirf_read_register(STATUS);
	  if ( conf & (1 << TX_FULL)){
		  //    ,  
		//flush tx fifo
		  nrf_print_status();
		  mirf_send_cmd(FLUSH_TX);
	  }

	//set tx mode
	mirf_CE_lo;
	//mirf_setTX;
	set_TX();
	_delay_us(210);



	//write data
	mirf_CSN_lo;
	spi_writeread( W_TX_PAYLOAD );
	for (uint8_t i = 0; i < data_lenght; i++)
		spi_writeread(data[i]);
	mirf_CSN_hi;

	//start transmission
	mirf_CE_hi;
	_delay_us(20);
	mirf_CE_lo;

	//wait for the transmission to stop
//	ACK_OK = 1;

	while(nrf24_isSending());	//,         
	ACK_OK=nrf24_lastMessageStatus();

	if (ACK_OK==2){
		//    ,     ,     
		//flush tx fifo
		mirf_send_cmd(FLUSH_TX);
	}
	mirf_write_register(STATUS,((1<<TX_DS)|(1<<MAX_RT)));	//  ,     

	//set rx mode
	mirf_setRX;
	_delay_us(135);
	mirf_CE_hi;
	return ACK_OK;
}

//       NRF (   payload)
void mirf_read_buf(uint8_t *data, uint8_t lenght)
{
	//read rx register
    mirf_CSN_lo;
    spi_writeread( R_RX_PAYLOAD );
    for (uint8_t i = 0; i < lenght; i++)
    	data[i] = spi_writeread(NOP);
    mirf_CSN_hi;
    //reset register
    mirf_write_register(STATUS,(1<<RX_DR));
}

//      FIFO  
uint8_t radio_read_rx_payload_width() {
  mirf_CSN_lo;
  spi_writeread(R_RX_PL_WID);
  uint8_t answ = spi_writeread(NOP);
  mirf_CSN_hi;
  return answ;
}

//  nrf   Standby
void mirf_power_up(void)
{
  uint8_t _config = spi_writeread(CONFIG);
  _config|=(1<<PWR_UP);						//  PWR_UP
  mirf_write_register(CONFIG, _config);
  _delay_ms(4);								//    
  //   PWR_UP    
  while(!(mirf_read_register(CONFIG) & _BV(PWR_UP)));
}

//  nrf  power down
void mirf_power_down(void)
{
	  uint8_t _config = mirf_read_register(CONFIG);
	  _config &= ~(1<<PWR_UP);					//  PWR_UP   nrf  
	  mirf_write_register(CONFIG, _config);

	  //   PWR_UP    
	  while(mirf_read_register(CONFIG) & _BV(PWR_UP));
}

void nrf_print_status(void)
{
uint8_t status=0;
status=mirf_get_status();
  printf_P(PSTR("STATUS\t\t = %02d RX_DR=%x TX_DS=%x MAX_RT=%x RX_P_NO=%x TX_FULL=%x\r"),
           status,
           (status & _BV(RX_DR))?1:0,
           (status & _BV(TX_DS))?1:0,
           (status & _BV(MAX_RT))?1:0,
           ((status >> RX_P_NO) & 0b111),
           (status & _BV(TX_FULL))?1:0
          );
}

void nrf_print_config(void)
{
uint8_t status=0;
status=mirf_read_register(CONFIG);
  printf_P(PSTR("CONFIG\t\t = %02d MASK_RX_DR=%x MASK_TX_DS=%x MASK_MAX_RT=%x EN_CRC=%x CRCO=%x PWR_UP=%x PRIM_RX=%x\r"),
           status,
        	(status & _BV(MASK_RX_DR))?1:0,
        	(status & _BV(MASK_TX_DS))?1:0,
            (status & _BV(MASK_MAX_RT))?1:0,
            (status & _BV(EN_CRC))?1:0,
            (status & _BV(CRCO))?1:0,
            (status & _BV(PWR_UP))?1:0,
            (status & _BV(PRIM_RX))?1:0
  );
}

void nrf_print_setup(void)
{
uint8_t status=0;
status=mirf_read_register(RF_SETUP);
  printf_P(PSTR("RF_SETUP\t\t = %02d CONT_WAVE=%x  RF_DR_LOW=%x PLL_LOCK=%x RF_DR_HIGH=%x RF_PWR=%x LNA_HCURR=%x\r"),
           status,
           (status & _BV(CONT_WAVE))?1:0,
           (status & _BV(RF_DR_LOW))?1:0,
           (status & _BV(PLL_LOCK))?1:0,
           (status & _BV(RF_DR_HIGH))?1:0,
 		   ((status >> RF_PWR) & 0b11),
           (status & _BV(LNA_HCURR))?1:0
        	  );
}
void nrf_print_future(void)
{
uint8_t status=0;
status=mirf_read_register(FEATURE);
  printf_P(PSTR("FEATURE\t\t = %02d EN_DPL=%x  EN_ACK_PAY=%x EN_DYN_ACK=%x\r"),
           status,
           (status & _BV(EN_DPL))?1:0,
           (status & _BV(EN_ACK_PAY))?1:0,
           (status & _BV(EN_DYN_ACK))?1:0
        	  );
}

void print_sprintf(const char* name,uint8_t reg){
	printf(name, reg);

}
//    NRF
void mirf_send_cmd(uint8_t reg)
{
	mirf_CSN_lo;
    spi_writeread(reg);
    mirf_CSN_hi;
}
