/**
 * \file timer.c
 * \brief  
 * \date 18 . 2015 .
 * \author: ARV
 */
 
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/atomic.h>
#include "config.h"
#include "timer.h"
#include "led.h"

extern config_t config;

///      
static volatile uint16_t repeat_delay;

///     
static volatile uint16_t wait_conversion;

///     (  ,      )
static volatile uint16_t wait_powerdown;

static uint8_t police_blink_delay[5]={2,2,2,2,2};	//      

static uint8_t array_pos=0;
static uint8_t curent_array_delay=0;

///        0  100.
uint8_t pwm_duty=0;

ISR(TIMER2_OVF_vect){
	//   2
	// 8000000/1024/255=30
	static uint8_t led_pos=0;

	if (curent_array_delay!=0){
		//     
		curent_array_delay--;
	}else{
		//         
		if (led_pos==0) {
			blink1_invert();
		}else {
			blink2_invert();
		}
		array_pos++;
		//          
		curent_array_delay=police_blink_delay[array_pos];
	}
	if (array_pos>sizeof(police_blink_delay)){
		//    
		array_pos=0;
		curent_array_delay=police_blink_delay[array_pos];
		//     
		if (led_pos==0) {
			led_pos=1;
		}else {
			led_pos=0;
		}
	}
}

ISR(TIMER1_OVF_vect){
//    1,        (1)
	TCNT1=TIMER1_PRESCALE;

	if(repeat_delay) repeat_delay--;
	if(wait_conversion) wait_conversion--;
	if (wait_powerdown) wait_powerdown--;
}

/**  2         
 * @param 
 * @return 
 */
void timer2_init(void){
	//  2
	TCNT2 = 0;
	//TCCR2 = 0;												/// Normal port operation, OC2A/OC2B disconnected, Normal Mode
	TCCR2 = _BV(CS20) | _BV(CS21) | _BV(CS22);				/// clkI/1024
	//ASSR |=_BV(AS2);										/// Asynchronous Timer/Counter2 for powerdown
	TIMSK |= _BV(TOIE2);									/// Timer/Counter2 Overflow Interrupt Enable
	//while (ASSR & (1<<TCN2UB | 1<<OCR2UB | TCR2UB)){}		//       
	//   ,    (     )
	//          
	array_pos=0;
	curent_array_delay=police_blink_delay[array_pos];
}

/**  2    powerdown
 * @param 
 * @return 
 */
void timer2_stop(void){
	//  2
	TCNT2 = 0;
	TCCR2 = 0;												/// Normal port operation, OC2A/OC2B disconnected, Normal Mode
	TIMSK &= ~_BV(TOIE2);									/// Timer/Counter2 Overflow Interrupt Disable
	//while (ASSR & (1<<TCN2UB | 1<<OCR2UB | TCR2UB)){}		//       
	blink1_off();
	blink2_off();
}

/**  2          powerdown
 * @param measuring_time        
 * @return 
 */
void timer1_init(void){
	//   16-bit Timer/Counter1
	TCNT1 = TIMER1_PRESCALE;
	TCCR1A = 0;												/// Normal port operation, OC1A/OC1B disconnected, Normal Mode
	TCCR1B |= _BV(CS12) | _BV(CS10);							/// clkI/O/1024 (From prescaler)
	TIMSK |= _BV(TOIE1);										/// Timer/Counter1, Overflow Interrupt Enable

	reset_timer(DELAY_AUTOSEND);
	reset_timer(DELAY_REPEAT);
	reset_timer(DELAY_POWERDOWN);
}

uint8_t time_is_over(delay_t t){
	switch(t){
	case DELAY_REPEAT:							///    
		ATOMIC_BLOCK(ATOMIC_RESTORESTATE){		///       
			return repeat_delay == 0;			///  1 -  repeat_delay=0,   0.
		}
		break;
	case DELAY_AUTOSEND:						///    
		ATOMIC_BLOCK(ATOMIC_RESTORESTATE){		///       
			return wait_conversion == 0;		///  1 -  wait_conversion=0,   0.
		}
		break;
	case DELAY_POWERDOWN:						///    
		ATOMIC_BLOCK(ATOMIC_RESTORESTATE){		///       
			return wait_powerdown == 0;			///  1 -  wait_powerdown=0,   0.
		}
		break;
	}
return 0;
}

void reset_timer(delay_t t){
	switch(t){
	case DELAY_REPEAT:
		ATOMIC_BLOCK(ATOMIC_RESTORESTATE){
			repeat_delay = 10;
		}
		break;
	case DELAY_AUTOSEND:
		ATOMIC_BLOCK(ATOMIC_RESTORESTATE){
			wait_conversion = config.delay_termo;
		}
		break;
	case DELAY_POWERDOWN:
			ATOMIC_BLOCK(ATOMIC_RESTORESTATE){
				//    300  (5 )
				wait_powerdown = 300;
			}
			break;
	}
}

/**  0     -
 * @param 
 * @return 
 */
void timer0_init(void){
	//  0
	TCCR0=0;
	TCNT0=0;
	TCCR0 |= _BV(CS00);										/// clkI(No prescaling)
	TIMSK |= _BV(TOIE0);									///  Timer/Counter0 Overflow Interrupt Enable
	DDR(PWM_PORT)|= _BV(PWM_PIN);		// enable as output
	pwm_off();
}

/**  0     -
 * @param 
 * @return 
 */
void timer0_stop(void){
	//  0
	TIMSK &= ~_BV(TOIE0);									/// Timer/Counter0 Overflow Interrupt Disable
	//while (ASSR & (1<<TCN2UB | 1<<OCR2UB | TCR2UB)){}		//       
	pwm_off();
}

ISR(TIMER0_OVF_vect){
//    0,      -
//    8000000/1/256=31250
	static uint8_t pwm_period=0;
	pwm_period++;
	if (pwm_period==0){
		//    
		pwm_on();
	}
	if (pwm_period==pwm_duty){
		//        
		pwm_off();
	}

}
